<?php
/**
 * ------------------------------------------------------------------------
 * JA Amazon S3 for Joomla 2.5 & Joomla 3.4
 * ------------------------------------------------------------------------
 * Copyright (C) 2004-2017 J.O.O.M Solutions Co., Ltd. All Rights Reserved.
 * @license - GNU/GPL, http://www.gnu.org/licenses/gpl.html
 * Author: J.O.O.M Solutions Co., Ltd
 * Websites: http://www.joomlart.com - http://www.joomlancers.com
 * ------------------------------------------------------------------------
 */

/**
 * @desc Modify from component Media Manager of Joomla
 *
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');

/**
 * Weblinks Component Weblink Model
 *
 * @package		Joomla
 * @subpackage	Content
 * @since 1.5
 */
class jaAmazonS3ModelRepo extends JModelLegacy
{

	function getState($property = null,$default = null)
	{
		$jinput = JFactory::getApplication()->input;
		$folder = $jinput->get( 'folder', '', 'STRING');
		//$folder = JPath::clean($folder);
		$this->setState('folder', $folder);

		$parent = str_replace("\\", "/", dirname($folder));
		$parent = ($parent == '.') ? null : $parent;
		$this->setState('parent', $parent);
		return parent::getState($property);
	}
	
	function getListServerFiles($bucket, $folder = '', $marker = '', $delimiter = '') {
		static $count = 0;
		$count++;
		
		$modelAccount = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
		$account = $modelAccount->getActiveAccount();
		
		$s3 = jaStorageHelper::getStorage_V2($account, $bucket->bucket_region);
		$aFiles = array();
		
		$maxKeys = 1000;
		if(!empty($folder)) {
			//upload for sub folder, not root folder
			$subFolderPath = jaStorageHelper::cleanPath($folder)."/";
		} else {
			$subFolderPath = $folder;
		}
		
		$opts = array("Prefix"=>$subFolderPath,"Marker"=>$marker, "Maxkeys" => $maxKeys);
		if(!empty($delimiter)) {
			$opts["Delimiter"] = $delimiter;
		}
		$opts["Bucket"] = $bucket->bucket_name;
		
		// Use the high-level iterators (returns ALL of your objects).
		try {
			$results = $s3->getPaginator('ListObjects', [
				'Bucket' => $opts['Bucket'],
				'Delimiter' => $delimiter,
				'Prefix' => $opts["Prefix"],
				'Marker' => $opts["Marker"],
				'Maxkeys' => $opts["Maxkeys"],
			]);

			foreach ($results->search('[Contents[]][]') as $file) {
				$fileName = (string) $file['Key']; 
				/**
				 * [0] => CFSimpleXML Object
						(
							[Key] => images/smilies/biggrin.gif
							[LastModified] => 2010-10-28T03:12:18.000Z
							[ETag] => "829dd9c5e7f55fdbcb34309ec44b973e"
							[Size] => 929
							[Owner] => CFSimpleXML Object
								(
									[ID] => 26466a41ee857b08763abe9568d29fb0078e62f5fb37a1cccb4399cf3e2e9638
									[DisplayName] => joomlart
								)

							[StorageClass] => STANDARD
						)
				 */
				//$aFiles[] = $file; 
				$aFiles[] = (object) array(
										'Key' => (string) $file['Key'], 
										'LastModified' => (string) $file['LastModified'],
										'ETag' => (string) $file['ETag'],  
										'Size' => (string) $file['Size']);
			}
// 			if(((string) $list['IsTruncated'] === 'true') && !empty($fileName) && ($fileName != $marker)) {
// 				//echo $marker."|".$fileName . "\r\n";
// 				/*if($count >= 3) {
// 					die();
// 				}*/
// 				//if result is truncated
// 				//use last file as marker to get next page
// 				$aFiles2 = $this->getListServerFiles($bucket, $folder, $fileName, $delimiter);
// 				//merge pages
// 				if(count($aFiles2)) {
// 					foreach ($aFiles2 as $file) {
// 						$aFiles[] = $file;
// 					}
// 				}
// 			}
		} catch (Aws\S3\Exception\S3Exception $e) {
			echo $e->getMessage() . "\n";
		}
		return $aFiles;
	}
	
	function updateAcl($account, $bucket, $acl = ACL_PRIVATE, $folder = '') {
		$modelBucket = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
		$oBucket = $modelBucket->getBucketByName($bucket);
		if(!$oBucket) {
			return false;
		}
		$s3 = jaStorageHelper::getStorage_V2($account, $oBucket->bucket_region);
		$list = $this->getListServerFiles($oBucket, $folder);
		// As long as we have at least one match...
		if (count($list) > 0)
		{
			// Hold CURL handles
			$handles = array();

			// Go through all of the items and delete them.
			foreach ($list as $item)
			{
				// Perform a batch of CopyObject operations.
				$handles[] = $s3->getCommand('putObjectAcl', array(
					'ACL'		=> $acl,
					'Bucket'	=> $bucket,
					'Key'		=> (string) $item->Key,
				));
			}
			
			try {
				$successful = $s3->execute($handles);
				$failed = false;
			} catch (\Guzzle\Service\Exception\CommandTransferException $e) {
				$successful = $e->getSuccessfulCommands();
				$failed = $e->getFailedCommands();
				return false;
			}
		}
		return true;
	}
	
	function getObjectAcl($list) {
		$acl = 'private';
		if(isset($list->Grant)) {
			foreach ($list->Grant as $user) {
				if(isset($user->Grantee) && isset($user->Grantee->URI)
					&& ($user->Grantee->URI == 'http://acs.amazonaws.com/groups/global/AllUsers')
				) {
					if(isset($user->Permission)) {
						switch ($user->Permission) {
							case 'FULL_CONTROL': $acl = 'open'; break;
							case 'WRITE': $acl = 'open'; break;
							case 'READ': $acl = 'public'; break;
						}
					} else {
						$acl = 'private';
					}
					break;
				}
			}
		}
		return $acl;
	}
	
	function deleteAdvance($bucket, $folder, $pcre = '\.php$') {
		$modelAccount = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
		$bucket = $this->getActiveBucket();
		$account = $modelAccount->getActiveAccount();

		$s3 = jaStorageHelper::getStorage_v2($account, $bucket->bucket_region);
		
		$folder = jaStorageHelper::cleanPath($folder);
		//delete is only apply for selected folder
		//escape Regex characters
		$folderSearch = preg_replace("/([\:\-\/\.\?\(\)\[\]\{\}])/", "\\\\$1", $folder);
		if(!empty($pcre)) {
			$pattern = "/{$folderSearch}.*?{$pcre}/";
			//$pattern = "/{$pcre}/";
		} else {
			//delete all
			//if want to delete all when pcre is blank, let remove above code check empty $pcre
			$pattern = "/{$folderSearch}.*/";
			//$pattern = "/.*/i";
		}
		
		$result = $s3->deleteMatchingObjects($bucket->bucket_name, $folder.'/', $pattern);
		return $result;
	}

	function getFolderTree($base = null)
	{
		$bucket = $this->getActiveBucket();
		if($bucket === false) {
			return array();
		}
		
		$modelAccount = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
		$account = $modelAccount->getActiveAccount();
		
		$s3 = jaStorageHelper::getStorage_V2($account, $bucket->bucket_region);
		
		$result = $s3->listObjects([
			// Bucket is required
			'Bucket' => $bucket->bucket_name,
			'Delimiter' => '/',
			'MaxKeys' => 5000,
			'Prefix' => '',
		]);
		
		$mediaBase = '';
		$folders = array();
		foreach ($result->search('[CommonPrefixes[]][]') as $item) {
			$folder = $item['Prefix'];
			// remove last slashes
			$folder = preg_replace("/\/*$/", "", $folder);
			if(!empty($folder) && $folder != '.' && $folder != '..') {
				$folder = "/{$folder}";
				if(!in_array($folder, $folders)) {
					$folders[] = $folder;
				}
			}
		}
		sort($folders);

		$tree = array();
		foreach ($folders as $folder)
		{
			$folder		= JPath::clean($folder);
			$folder		= str_replace('\\', '/', $folder);
			$name		= substr($folder, strrpos($folder, '/') + 1);
			$relative	= str_replace($mediaBase, '', $folder);
			$absolute	= $folder;
			$path		= explode('/', $relative);
			$node		= (object) array('name' => $name, 'relative' => $relative, 'absolute' => $absolute);

			$tmp = &$tree;
			for ($i=0,$n=count($path); $i<$n; $i++)
			{
				if (!isset($tmp['children'])) {
					$tmp['children'] = array();
				}
				if ($i == $n-1) {
					// We need to place the node
					$tmp['children'][$relative] = array('data' =>$node, 'children' => array());
					break;
				}
				if (array_key_exists($key = implode('/', array_slice($path, 0, $i+1)), $tmp['children'])) {
					$tmp = &$tmp['children'][$key];
				}
			}
		}
		$tree['data'] = (object) array('name' => $bucket->bucket_name, 'relative' => '', 'absolute' => $base);
		return $tree;
	}
	
	function _getVars_admin() {
		$mainframe = JFactory::getApplication('administrator');
		$option='repo';
		$lists = array ();
		$lists ['bucket_id'] = $mainframe->getUserStateFromRequest ( $option . '.bucket_id', 'bucket_id', 0, 'int' );
		return $lists;
	}	
	
	function getActiveBucket($bucket_id = 0) {
		if(!$bucket_id) {
			$lists = $this->_getVars_admin();
			$bucket_id = $lists['bucket_id'];
		}
		
		if(!$bucket_id) {
			return false;
		} else {
			$modeBucket = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
			$bucket = $modeBucket->getActiveBucket($bucket_id);
			
			if(!isset($bucket->id) || !$bucket->id) {
				return false;
			} else {
				return $bucket;
			}
		}
	}
}