<?php
/**
 * ------------------------------------------------------------------------
 * JA Amazon S3 for Joomla 2.5 & Joomla 3.4
 * ------------------------------------------------------------------------
 * Copyright (C) 2004-2017 J.O.O.M Solutions Co., Ltd. All Rights Reserved.
 * @license - GNU/GPL, http://www.gnu.org/licenses/gpl.html
 * Author: J.O.O.M Solutions Co., Ltd
 * Websites: http://www.joomlart.com - http://www.joomlancers.com
 * ------------------------------------------------------------------------
 */
/**
 * @desc Modify from component Media Manager of Joomla
 *
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

/**
 * Media Component List Model
 *
 * @package		Joomla
 * @subpackage	Media
 * @since 1.5
 */
class jaAmazonS3ModelRepolist extends JModelLegacy
{

	function getState($property = null, $default = null)
	{
		$jinput = JFactory::getApplication()->input;
		$folder = $jinput->get( 'folder', '', 'STRING');
		//$folder = JPath::clean($folder);
		$this->setState('folder', $folder);

		$parent = str_replace("\\", "/", dirname($folder));
		$parent = ($parent == '.') ? null : $parent;
		$this->setState('parent', $parent);
		return parent::getState($property, $default);
	}

	function getImages()
	{
		$list = $this->getList();
		return $list['images'];
	}

	function getFolders()
	{
		$list = $this->getList();
		return $list['folders'];
	}

	function getDocuments()
	{
		$list = $this->getList();
		return $list['docs'];
	}

	/**
	 * Build imagelist
	 *
	 * @param string $listFolder The image directory to display
	 * @since 1.5
	 */
	function getList()
	{
		static $list;
		// Only process the list once per request
		if (is_array($list)) {
			return $list;
		}
		
		$modelRepo = JModelLegacy::getInstance ( 'repo', 'jaAmazonS3Model' );
		$bucket = $modelRepo->getActiveBucket();
		if($bucket === false) {
			return array('folders' => array(), 'docs' => array(), 'images' => array());
		}
		
		$modelAccount = JModelLegacy::getInstance ( 'bucket', 'jaAmazonS3Model' );
		$account = $modelAccount->getActiveAccount();
		
		$s3 = jaStorageHelper::getStorage_V2($account, $bucket->bucket_region);


		// Get current path from request
		$current = $this->getState('folder');

		// If undefined, set to empty
		if ($current == 'undefined') {
			$current = '';
		}
		
		$current = jaStorageHelper::cleanPath($current);
		if(!empty($current)) {
			$current .= "/";
			$deepLevel = count(explode("/", $current)) - 1;
		} else {
			$deepLevel = 0;
		}
		
		// Use the high-level iterators (returns ALL of your objects).
		try {
			$results = $s3->getPaginator('ListObjects', [
				'Bucket' => $bucket->bucket_name,
				'Delimiter' => '/', // not fetch data on this path => / to make current path
				'Prefix' => $current // only fetch data from this path
			]);
		} catch (S3Exception $e) {
			echo $e->getMessage() . "\n";
			return array();
		}
		
		$images 	= array ();
		$folders 	= array ();
		$docs 		= array ();
		
		// Initialize variables
		$basePath = $current;
		$mediaBase = '';

		if(isset($results)) {
			foreach ($results as $result) {
				// Get the list of files and folders from the given folder
				$fileList 	= !empty($result['Contents']) ? $result['Contents'] : array();
				if (!empty($result['CommonPrefixes']))
					foreach ($result['CommonPrefixes'] AS $prefix) {
						$fileList[] = $prefix;
					}

				$iconUrl = "components/".JACOMPONENT."/assets/images/icons/";
				$iconPath = JPath::clean(JPATH_ADMINISTRATOR.'/'.$iconUrl.'/');
				// Iterate over the files if they exist
				if ($fileList !== false) {
					foreach ($fileList as $file)
					{
						/*$acl = $s3->get_object_acl($bucket->bucket_name, $file->Key);
						if($acl) {
							$status = $modelRepo->getObjectAcl($acl->body->AccessControlList);
						} else {
							$status = 'na';
						}*/
						$status = 'na';
						$fileKey = isset($file['Prefix']) ? (string) $file['Prefix'] : (string) $file['Key'];
					
						// detect folder.
						if (isset($file['Prefix'])) {
							$tmp = new JObject();
							$tmp->name = basename($fileKey);
							$tmp->path = str_replace('\\', '/', JPath::clean($fileKey));
							$tmp->path_relative = str_replace($mediaBase, '', $tmp->path);
							$tmp->files = 0;
							$tmp->folders = 0;
							$folders[] = $tmp;
							continue; // break if folder.
						}
					
						$deepLevelSub = count(explode("/", $fileKey));
						if (substr($fileKey, -1) != '/' && substr($fileKey, 0, 1) != '.' && strtolower($fileKey) !== 'index.html' && ($deepLevelSub == $deepLevel + 1) ) {
							$tmp = new JObject();
							$tmp->name = basename($fileKey);
							$tmp->path = str_replace('\\', '/', JPath::clean($fileKey));
							$tmp->path_relative = str_replace($mediaBase, '', $tmp->path);
							$tmp->size = $file['Size'];
							$tmp->lastModified = $file['LastModified'];
							$tmp->status = $status;
							$ext = strtolower(JFile::getExt($fileKey));
							$tmp->ext = $ext;
							switch ($ext)
							{
								// Image
								case 'jpg':
								case 'png':
								case 'gif':
								case 'xcf':
								case 'odg':
								case 'bmp':
								case 'jpeg':
									$iconfile_32 = $iconPath."mime-icon-32/".$ext.".png";
									if (file_exists($iconfile_32)) {
										$tmp->icon_32 = $iconUrl."mime-icon-32/".$ext.".png";
									} else {
										$tmp->icon_32 = $iconUrl."con_info.png";
									}
									$iconfile_16 = $iconPath."mime-icon-16/".$ext.".png";
									if (file_exists($iconfile_16)) {
										$tmp->icon_16 = $iconUrl."mime-icon-16/".$ext.".png";
									} else {
										$tmp->icon_16 = $iconUrl."con_info.png";
									}
									$images[] = $tmp;
									break;
								// Non-image document
								default:
									$iconfile_32 = $iconPath."mime-icon-32/".$ext.".png";
									if (file_exists($iconfile_32)) {
										$tmp->icon_32 = $iconUrl."mime-icon-32/".$ext.".png";
									} else {
										$tmp->icon_32 = $iconUrl."con_info.png";
									}
									$iconfile_16 = $iconPath."mime-icon-16/".$ext.".png";
									if (file_exists($iconfile_16)) {
										$tmp->icon_16 = $iconUrl."mime-icon-16/".$ext.".png";
									} else {
										$tmp->icon_16 = $iconUrl."con_info.png";
									}
									$docs[] = $tmp;
									break;
							}
						}
					}
				}
			}
		}
		
		$list = array('folders' => $folders, 'docs' => $docs, 'images' => $images);
		//adding upload log information
		$db = JFactory::getDBO();
		foreach ($list as $group => $items) {
			if(count($items)) {
				foreach ($items as $id => $item) {
					$sql = "SELECT last_update, file_original_checksum FROM `#__jaamazons3_file` WHERE file_exists = 1 AND bucket_id = '{$bucket->id}' AND path = ".$db->quote($item->path_relative);
					$db->setQuery($sql);
					$log = $db->loadObject();
					if(is_object($log)) {
						$item->last_update = $log->last_update;
						$item->compressed = !empty($log->file_original_checksum) ? 1 : 0;
					} else {
						$item->last_update = false;
						$item->compressed = 0;
					}
					$list[$group][$id] = $item;
				}
			}
		}

		return $list;
	}
}